% Computes the overall accuracy OA and kappa index using a linear
% classifier. The estimated labels are obtained by projected data onto
% principal components of KOPLS method with different kernels(Ks: RBF kernel,
% Kc:cluster kernel and linear combination of them: K = alphas*Ks+(1-alphas)*Kc).
%
% Inputs:
%       -X      : Original labeled data. Matrix, M(samples)xF(features).
%       -Xu     : Original unlabeled data.  Matrix, N(samples)xF(features).
%       -Y      : Labels of X.  Vector, M(samples)x1. (It's only use to predict.)
%       -Yu     : Labels of Xu. Vector, M(samples)x1. (It's only use to testing.)
%       -Kbag   : Cluster kernel. Matrix, M(samples)xM(samples).
%       -KbagUL : Test cluster kernel. Matrix, M(train samples)xN(test samples).
%       -C_tot  : Strut containing centers and mixing components of the Gaussians.
%       -IDX_tot: Strut containing partitions data determined by GMM.
%       -alphas : K = alphas*Ks+(1-alphas)*Kc.
%
% Outputs:
%       -OA   : Overall acuracy. Vector, 1XF(features).
%       -Kappa: Estimated Cohen's Kappa coefficient. Vector, 1XF(features).

function [OA_SUMA,Kappa]=Assesskopls(X,Xu,Y,Yu,Kbag,KbagUL,Centroides,IDX,alphas)

N = 20;  % #realizations of gmm
k = 2:20; % number of clusters

%Centered labels
Yb = binarize(Y); Yb = Yb - ones(length(Y),1)*mean(Yb);

% Rough estimation of the sigma parameter
sigmax = estimateSigma(X,X);

disp('Step 8: Training KOPLS ...')
%%%%SUPERVISED CASE%%%%

Ktrains  = kernelmatrix('rbf',X',X',sigmax);
Ktrain=kernelcentering(Ktrains);
Ktest  = kernelmatrix('rbf',X',Xu',sigmax);
Ktest=kernelcentering(Ktest,sum(Ktrains));

figure, imagesc(Ktrain)
axis off equal
title('Kernel KOPLS')

% KOPLS for fextractor.n_p features
fextractor.centroids = Ktrain;
fextractor = isp_rkopls_phase1(Ktrain,Yb,fextractor);
fextractor.alpha = gen_eig(fextractor.KrY*fextractor.KrY',fextractor.B,fextractor.n_p);

%Data projection
Xkopls = isp_rkopls_phase3(Ktrain,fextractor);
Xkopls2 = isp_rkopls_phase3(Ktest,fextractor);

figure,
for t = 1:fextractor.n_p
    B = inv(Xkopls(:,1:t)'*Xkopls(:,1:t)) * (Xkopls(:,1:t)' * Yb);
    test = Xkopls2(:,1:t) * B;
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on, subplot(1,2,t),plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,1,fextractor.alpha(:,1:t),Y,Yu,t)
        title(strcat('KOPLS feature:',num2str(t)))
    end
    
    %Over Accuracy of KOPLS in test
    RESkopls = assessment(Yu,Ypred,'class');
    OA_SUMA_sup(t)    = RESkopls.OA;
    Kappa_sup(t)      = RESkopls.Kappa;
end
disp('Done!')

if size(Xkopls,2)>1
    figure_projections(Xkopls(:,1:2),Xkopls2(:,1:2),Y,Yu)
    xlabel('Feature 1')
    ylabel('Feature 2')
    axis off
    title('Projected data onto the first 2 directions of KOPLS')
    legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')
end

disp('Step 9: Training KcOPLS ...')
%%%NON SUPERVISED CASE%%%%

Ktrains = Kbag;
Ktrain=kernelcentering(Ktrains);
Ktest      = KbagUL;
Ktest=kernelcentering(Ktest,sum(Ktrains));

figure, imagesc(Ktrain)
axis off equal
title('Cluster Kernel')

%KOPLS for fextractor.n_p features
fextractor.centroids = Ktrain;
fextractor = isp_rkopls_phase1(Ktrain,Yb,fextractor);
fextractor.alpha = gen_eig(fextractor.KrY*fextractor.KrY',fextractor.B,fextractor.n_p);

% Data projections
Xkopls = isp_rkopls_phase3(Ktrain,fextractor);
Xkopls2 = isp_rkopls_phase3(Ktest,fextractor);

figure,
for t = 1:fextractor.n_p
    B = inv(Xkopls(:,1:t)'*Xkopls(:,1:t)) * (Xkopls(:,1:t)' * Yb);
    test = Xkopls2(:,1:t) * B;
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on, subplot(1,2,t),plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,0,fextractor.alpha(:,1:t),Y,Yu,t)
        title(strcat('Kc feature:',num2str(t)))
    end
    
    %Over Accuracy of KOPLS in test
    RESkoplns = assessment(Yu,Ypred,'class');
    OA_SUMA_ns(t)    = RESkoplns.OA;
    Kappa_ns(t)      = RESkoplns.Kappa;
end
disp('Done!')

if size(Xkopls,2)>1
    figure_projections(Xkopls(:,1:2),Xkopls2(:,1:2),Y,Yu)
    xlabel('Feature 1')
    ylabel('Feature 2')
    axis off
    title('Projected data onto the first 2 directions of the Cluster kernel Kc')
    legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')
end

disp('Step 10: Training KcOPLS ...')
%%%%SEMISUPERVISED CASE%%%%

% Build Semi-upervised kernel:
Ktrain1  = kernelmatrix('rbf',X',X',sigmax);
Ktest1  = kernelmatrix('rbf',X',Xu',sigmax);
Ktrains = alphas*Ktrain1+(1-alphas)*Kbag;
Ktrain=kernelcentering(Ktrains);
Ktest      = alphas*Ktest1 + (1-alphas)*KbagUL;
Ktest=kernelcentering(Ktest,sum(Ktrains));

%KOPLS for fextractor.n_p features
fextractor.centroids = Ktrain;
fextractor = isp_rkopls_phase1(Ktrain,Yb,fextractor);
fextractor.alpha = gen_eig(fextractor.KrY*fextractor.KrY',fextractor.B,fextractor.n_p);

%Data projections
Xkopls = isp_rkopls_phase3(Ktrain,fextractor);
Xkopls2 = isp_rkopls_phase3(Ktest,fextractor);

figure,
for t=1:fextractor.n_p
    B = inv(Xkopls(:,1:t)'*Xkopls(:,1:t)) * (Xkopls(:,1:t)' * Yb);
    test = Xkopls2(:,1:t) * B;
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on,subplot(1,2,t), plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,alphas,fextractor.alpha(:,1:t),Y,Yu,t)
        title(strcat('SS-KOPLS, feature:',num2str(t)))
    end
    
    %Over Accuracy of KOPLS in test
    RESkoplss = assessment(Yu,Ypred,'class');
    OA_SUMA_ss(t)    = RESkoplss.OA;
    Kappa_ss(t)      = RESkoplss.Kappa;
end
disp('Done!')

if size(Xkopls,2)>1
    figure, imagesc(Ktrain)
    axis off equal
    title('Kernel SS-KOPLS')
    figure_projections(Xkopls,Xkopls2,Y,Yu)
    axis off
    xlabel('Feature 1')
    ylabel('Feature 2')
    title('Projected data onto the first 2 directions of SS-KOPLS')
    legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')
end

OA_SUMA=[OA_SUMA_ns;OA_SUMA_ss;OA_SUMA_sup];
Kappa=[Kappa_ns;Kappa_ss;Kappa_sup];