% Computes the overall accuracy OA and kappa index using a linear
% classifier. The estimated labels are obtained by projected data onto
% principal components of KPLS method with different kernels(Ks: RBF kernel,
% Kc:cluster kernel and linear combination of them: K = alphas*Ks+(1-alphas)*Kc).
%
% Inputs:
%       -X      : Original labeled data. Matrix, M(samples)xF(features).
%       -Xu     : Original unlabeled data.  Matrix, N(samples)xF(features).
%       -Y      : Labels of X.  Vector, M(samples)x1. (It's only use to predict.)
%       -Yu     : Labels of Xu. Vector, M(samples)x1. (It's only use to testing.)
%       -Kbag   : Cluster kernel. Matrix, M(samples)xM(samples).
%       -KbagUL : Test cluster kernel. Matrix, M(train samples)xN(test samples).
%       -C_tot  : Strut containing centers and mixing components of the Gaussians.
%       -IDX_tot: Strut containing partitions data determined by GMM.
%       -alphas : K = alphas*Ks+(1-alphas)*Kc.
%       -Nfmax  : # features extracted.
%
% Outputs:
%       -OA   : Overall acuracy. Vector, 1XF(features).
%       -Kappa: Estimated Cohen's Kappa coefficient. Vector, 1XF(features).

function [OA_SUMA,Kappa]=Assesskpls(X,Xu,Y,Yu,Kbag,KbagUL,Centroides,IDX,alphas,Nfmax)

N = 20;  % #realizations of gmm
k = 2:20; % number of clusters

%Center labels
Yb = binarize(Y); Yb = Yb - ones(length(Y),1)*mean(Yb);

% Rough estimation of the sigma parameter
sigmax = estimateSigma(X,X);

NFKERNELS=1:Nfmax;

disp('Step 5: Training KPLS ...')
%%%%SUPERVISED CASE%%%%
Ktrains  = kernelmatrix('rbf',X',X',sigmax);
Ktrain=kernelcentering(Ktrains);
Ktest  = kernelmatrix('rbf',X',Xu',sigmax);
Ktest=kernelcentering(Ktest,sum(Ktrains));

figure, imagesc(Ktrain)
axis off equal
title('Kernel KPLS')

figure,
for t = NFKERNELS %Loop for number features extract
    
    %KPLS for t features
    [alpha2,test,beta] = dualpls(Ktrain,Ktest,Yb,t);
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on, subplot(1,2,t),plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,1,beta,Y,Yu,t)
        title(strcat('KPLS feature:',num2str(t)))
    end
    
    %Overall Accuracy of KPLS in test set
    RESkpls = assessment(Yu,Ypred,'class');
    OA_SUMA_sup(t)    = RESkpls.OA;
    Kappa_sup(t)    = RESkpls.Kappa;
end
disp('Done!')

Xkpls=Ktrain*beta(:,1:2);
Xkpls2=Ktest'*beta(:,1:2);
figure_projections(Xkpls,Xkpls2,Y,Yu)
xlabel('Feature 1')
ylabel('Feature 2')
axis off
title('Projected data onto the first 2 directions of KPLS')
legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')

disp('Step 6: Training KcPLS ...')
%%%NON SUPERVISED CASE%%%%
Ktrains = Kbag;
Ktrain=kernelcentering(Ktrains);
Ktest      = KbagUL;
Ktest=kernelcentering(Ktest,sum(Ktrains));

figure, imagesc(Ktrain)
axis off equal
title('Cluster Kernel')

figure,
for t = NFKERNELS %Loop for number features extract
    
    %KPLS for t features
    [alpha2,test,beta] = dualpls(Ktrain,Ktest,Yb,t);
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on, subplot(1,2,t),plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,0,beta,Y,Yu,t)
        title(strcat('Kc feature:',num2str(t)))
    end

    %Over Accuracy of KPLS in test
    RESkpls = assessment(Yu,Ypred,'class');
    OA_SUMA_ns(t)    = RESkpls.OA;
    Kappa_ns(t)    = RESkpls.Kappa;
end
disp('Done!')

Xkpls=Ktrain*beta(:,1:2);
Xkpls2=Ktest'*beta(:,1:2);
figure_projections(Xkpls,Xkpls2,Y,Yu)
xlabel('Feature 1')
ylabel('Feature 2')
axis off
title('Projected data onto the first 2 directions of the Cluster kernel Kc')
legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')

disp('Step 7: Training SSKPLS ...')
%%%%SEMISUPERVISED CASE%%%%

% Build Semi-upervised kernel:
Ktrain1  = kernelmatrix('rbf',X',X',sigmax);
Ktest1  = kernelmatrix('rbf',X',Xu',sigmax);
Ktrains = alphas*Ktrain1+(1-alphas)*Kbag;
Ktrain=kernelcentering(Ktrains);
Ktest      = alphas*Ktest1 + (1-alphas)*KbagUL;
Ktest=kernelcentering(Ktest,sum(Ktrains));

figure, imagesc(Ktrain)
axis off equal
title('Kernel SS-KPLS')

figure,
for t=NFKERNELS
    
    %KPLS for t features
    [alpha2,test,beta] = dualpls(Ktrain,Ktest,Yb,t);
    [val,Ypred]    = max(test,[],2);
    
    if t==1||t==2
        hold on,subplot(1,2,t), plotKernelFeatures(X,Xu,Centroides,IDX,sigmax,N,k,alphas,beta,Y,Yu,t)
        title(strcat('SS-KPLS, feature:',num2str(t)))
    end
    
    %Over Accuracy of KPLS in test
    RESkpls = assessment(Yu,Ypred,'class');
    OA_SUMA_ss(t)    = RESkpls.OA;
    Kappa_ss(t)    = RESkpls.Kappa;
end
% end
disp('Done!')

Xkpls=Ktrain*beta(:,1:t);
Xkpls2=Ktest'*beta(:,1:t);
figure_projections(Xkpls,Xkpls2,Y,Yu)
xlabel('Feature 1')
ylabel('Feature 2')
axis off
title('Projected data onto the first 2 directions of SS-KPLS')
legend('Unlabeled data, class 1','Unlabeled data, class 2','Labeled data, class 1','Labeled data, class 2')

OA_SUMA=[OA_SUMA_ns;OA_SUMA_ss;OA_SUMA_sup];
Kappa=[Kappa_ns;Kappa_ss;Kappa_sup];


