function fextractor = isp_rkopls_phase1(K,Y,fextractor)

% @description
% Implements the OPLS method for FE projecting the input data
% either in original space or in feature space (KOPLS).
% Phase 1 : Sequential computation of the required matrices
%
% @function
% fextractor = isp_rkopls_phase1(X,Y,fextractor)
%
% @input
% X: matrix of input data (lx x N). Each row is a pattern
% Y: matrix of desired targets (lx x M). Each row contains all the targets
%    for each pattern. 1-out-of-M encoding for classification
% fextractor: struct with all parameters for computing features
%
% @output
% fextractor: struct with all parameters for computing features
%
% Fields of the fextractor structure
%
% 1) Fields that must be initialized when calling this function
%   - fextractor.meanX : 1 x N vector, contains the estimated mean for each
%                        input variable
%   - fextractor.std : 1 x N vector, contains the estimated standard
%                      deviation for each input dimension
%   - fextractor.meanY : 1 x M vctor, contains the estimated mean for each
%                        output variable, i.e., in classification, a priori
%                        distribution for each class
%   - fextractor.ker : For backward compatibility. In this function, we
%                      will use fextractor.ker = 'rbf'
%   - fextractor.par : With of Gaussian parameter
%   - fextractor.centroids : R x N matrix of CENTERED input data. This data
%                            needs to be centered with fextractor.meanX and
%                            fextractor.std outside this function
%
% 2) Fields that are initialized/updated by this function
%   - fextractor.B : R x R matrix containing the partial KR x KR^T
%   - fextractor.KrY : R x M matrix containing the partial KR x Y
%   - fextractor.cent_vector : For centering of data in feature space
%   - fextractor.lx : Number of input samples considered up to now
%
% 3) Other fields not used by this function
%   - fextractor.n_p : Number of projections/features extracted
%   - fextractor.alpha : R x n_p matrix containing the coefficients for
%                        each projection
%
% @copyrights Intelligent sound 2006
% @licence GPL?
% @author Jer�nimo Arenas-Garc�a, ISP,IMM,DTU
% @date 29-06-2006
%
% @modified_by XXX
% @modified_date XXX
% @modified_log XXX
%
% @version 1.0
%
% @examples
% fextractor = isp_rkopls_phase1(X,Y,fextractor)

if (nargin < 3) % check correct number of arguments
    error('Check correct number of arguments')
end

[lx,lx1] = size(K);
[ly,M] = size(Y);

if isfield(fextractor,'R')
    R=fextractor.R;
else
    [R,N] = size(fextractor.centroids);
end

if (lx~=ly)
    error('The number of rows in X and Y must be the same')
end

% Normalization of the Input and Output Data
% X = (X-ones(lx,1)*fextractor.meanX)./(ones(lx,1)*fextractor.std);
% Y = (Y-ones(lx,1)*fextractor.meanY);

% Checking and initializing the optional fields
if (~isfield(fextractor,'ker')) fextractor.ker = 'rbf'; end
% if (~isfield(fextractor,'par')) fextractor.par = sqrt(N/2); end

if (~isfield(fextractor,'B')) fextractor.B = zeros(R); end
if (~isfield(fextractor,'KrY')) fextractor.KrY = zeros(R,M); end
if (~isfield(fextractor,'lx')) fextractor.lx = 0; end
if (~isfield(fextractor,'cent_vector')) fextractor.cent_vector = zeros(R,1); end
if isfield(fextractor,'R')
    m=randperm(lx);
    Kernel_train=K(m(1:R),:);
    pkk=Kernel_train;
else
    pkk=K;
end

fextractor.B = pkk*pkk';
fextractor.KrY= pkk *Y;
% %  for k = 1:lx
% %     pkk=Kernel_train(:,k);
% % %     fextractor.cent_vector = fextractor.cent_vector + pkk;
% %     fextractor.B = fextractor.B + pkk * pkk';
% %     fextractor.KrY = fextractor.KrY + pkk * Y(k,:);
% % end

fextractor.lx = fextractor.lx + lx;

if (isfield(fextractor,'n_p'))
    fextractor.n_p = min([fextractor.n_p R rank(fextractor.KrY)]);
else
    fextractor.n_p = min([R rank(fextractor.KrY)]);
end
