% Builds a GMM cluster kernel given input labeled and unlabeled data
% Inputs:
%       - Xtrain     : Original labeled data. Matrix,   M(samples)xF(features).
%       - Xunlabeled : Original unlabeled data. Matrix, N(samples)xF(features).
%       - N          : # iterations of GMM.
%       - Kvector    : # maximum cluster of GMM
%
% Outputs:
%       - Kbag    : Cluster kernel. Matrix, M(samples)xM(samples).
%       - C_tot   : Strut containing centers and mixing components of the Gaussians.
%       - IDX_tot : Strut containing partitions data determined by GMM.

function [Kbag,C_tot,IDX_tot]=bagkernelgmm(Xtrain,Xunlabeled,N,Kvector)


Xtest=Xunlabeled;

% addedPts is a parameter to tune if we want to add test points to the
% GMM estimation (in case we do not have enough train pixels)
addedPts = 1000; % maximum # of test samples in order to build cluster kernel
if addedPts>size(Xtest,1)
    addedPts=size(Xtest,1);
end

c = randperm(size(Xtest,1))';
Kbag = zeros(size(Xtrain,1)+addedPts);
IDX_tot = cell(length(Kvector),N);
C_tot = cell(length(Kvector),N);
SIG_tot = cell(length(Kvector),N);

nk=0;
for k = Kvector
    nk = nk+1;
    for n=1:N
        disp(['Building the cluster/generative kernel for (k,n)=' num2str(k) ',' num2str(n)])
        try
            options = statset('Display','final');
            % GMM
            obj = gmdistribution.fit([Xtrain; Xtest(c(1:addedPts),:)],k,'Options',options);
            % Posterior probability of components
            Pgmm = posterior(obj,[Xtrain; Xtest(c(1:addedPts),:)]);
            %  Partitions data
            IDX = cluster(obj,[Xtrain; Xtest(c(1:addedPts),:)]);
            % Inner product of posterior probability
            PP = Pgmm*Pgmm';
            
            % Bag kernel from GMM clustering
            for i=1:(size(Xtrain,1)+addedPts)
                for j=1:(size(Xtrain,1)+addedPts)
                    Kbag(i,j) = Kbag(i,j) + PP(i,j);
                end;
            end;
            % Structs of partition data and obj of GMM
            IDX_tot{nk,n} = IDX;
            C_tot{nk,n} = obj;
            
            clear IDX
            clear C
        catch
            disp('GMM failed due to low samples/dim ratio')
        end
    end
end

%-------------------------------------------------------------------------
% Normalization of cluster kernel
%-------------------------------------------------------------------------
Kbag = Kbag(1:size(Xtrain,1),1:size(Xtrain,1));
Z=max(max(Kbag));
Kbag = Kbag/Z;

